# Class: safeshare::access_service
#
# This class manages the SafeShare Access Service
#
# Parameters:
#
#  [*version*]
#    Optional. Default value is `latest`.
#    The SafeShare Access Service version to install.
#    Should be version 3.0 and above. Should include the full
#    version number (e.g. '3.0-965-1'). Can also be any possible
#    `Package` resource `ensure` value such as `latest` or `present`
#
#  [*custom_app_source_path*]
#    Optional source directory containing customized files i.e. email notification
#    templates.
#
#  [*custom_notifications_source_path*]
#    Optional source directory containing
#    [FreeMarker Templates](http://freemarker.org/)
#
#  [*database_hostname*]
#    Hostname of the PostgresQL database.
#
#  [*database_username*]
#    Username used by the application to connect to PostgresQL.
#
#  [*database_password*]
#    Password used by the application to connect to PostgresQL.
#
#  [*access_service_url*]
#    The base URL on which the SafeShare instance is served.
#
#  [*content_service_url*]
#    The base URL which is used to refer to the Content Service.
#
#  [*webapp_service_url*]
#    The base URL of the Safe Share webapp resources service.
#    If you intend to use the default port value of 80, then do not explicitly
#    specify the port value 80 in this base URL parameter's value. Otherwise,
#    you will not be able to access the login page.
#
#  [*mail_auth*]
#    A boolean flag telling whether the SMTP server requires authentication
#    (`true`) or not (`false`).
#    If this is `false` then `mail_password` and `mail_username` are ignored.
#
#  [*mail_password*]
#    Password to use for authentication to the SMTP server. Used only if
#    `mail_auth` is set to `true`.
#
#  [*mail_username*]
#    User to use for authentication to the SMTP server. Used only if
#    `mail_auth` is set to `true`.
#
#  [*mail_hostname*]
#    FQDN or IP address of the SMTP server.
#
#  [*mail_ssl_enabled*]
#    A boolean flag telling whether the SMTP server uses SSL protocol or not.
#    Note: If this value is true then *mail_starttls_enabled* cannot be true.
#
#  [*mail_starttls_enabled*]
#    A boolean flag telling whether should use STARTTLS.
#    Note: If this value is true then *mail_ssl_enabled* cannot be true.
#
#  [*mail_port*]
#    Port number to connect to the SMTP server (typically 25 for plain SMTP
#    or STARTTLS and 587 if using SMTP over SSL).
#
#  [*mail_from_address*]
#    E-mail address that the e-mails which are automatically generated by
#    Access Service appear to be coming from. This is where e-mails will
#    bounce to in case of error.
#
#  [*java_xmx*]
#    Optional. Java VM's `-Xmx` parameter (maximum size of memory allocation
#    pool in bytes).
#    Default is `2G`. May have to be increased if you are handling very large
#    files.
#
#  [*min_firefox_version*]
#    (Optional). Oldest version of Firefox supported by the software.
#
#  [*company_name*]
#    (Optional). Name of company owning the SafeShare server. It's used in the
#    Subject and From headers of e-mail.
#
#  [*master_encryption_key*]
#    Base64 encoded string containing a 256-bit random AES key.
#    It's used to encrypt all cryptographic keys which are stored in the
#    database.
#    If this key is lost or changed, all existing data will become inaccessible.
#    Example command for generating the random key using `openssl`:
#
#    `openssl rand -base64 32`
#
#  [*bearer_signing_key*]
#    Shared secret used to sign messages between the Access Service and the
#    Content Service during the initial handshake for generating a JSON
#    Web Token (JWT).
#    It must have the same value as
#    `$safeshare::content_service::bearer_signing_key`
#    It is a string of any random bytes.
#    Example command for generating it using `openssl`:
#
#    `openssl rand -base64 20`
#
#  [*jwt_signing_key*]
#    A private key used to sign the JSON Web Token (JWT)
#    between Access Service and Content Service.
#    It is a hex encoded string of random bytes.
#    Example command for generating it using `openssl`:
#
#    `openssl rand -hex 20`
#
#  [*jwt_rsa_private_key*]
#    Puppet path of the private RSA key used to sign the JSON Web Token (JWT)
#    between Access Service and clients.
#    http://docs.puppetlabs.com/puppet/latest/reference/type.html#file-attribute-source
#    Example: `puppet:///modules/profiles/safeshare/access/jwt/jwt_public_key.pem`
#
#    Example command for generating the key using `openssl`:
#    `openssl genrsa -out jwt_private_key.pem 2048`
#
#  [*jwt_rsa_public_key*]
#    Puppet path of the public RSA key used to sign the JSON Web Token (JWT)
#    between Access Service and clients.
#    This is the public key of `jwt_rsa_private_key`.
#    http://docs.puppetlabs.com/puppet/latest/reference/type.html#file-attribute-source
#    Example: `puppet:///modules/profiles/safeshare/access/jwt/jwt_public_key.pem`
#
#    Example command for generating the key using `openssl`:
#    `openssl rsa -in jwt_private_key.pem -pubout -out jwt_public_key.pem`
#
#  [*http_port*]
#    (Optional) HTTP port which the Java applications listens on.
#    Default is `8080`.
#
#  [*rabbitmq_user_name*]
#    Name of user used to access the RabbitMQ service.
#
#  [*rabbitmq_password*]
#    Password used to access the RabbitMQ service.
#
#  [*rabbitmq_host*]
#    Name or IP address of host of RabbitMQ service.
#
#  [*rabbitmq_port*]
#    (Optional) Port which the RabbitMQ service listens on. Default is `5672`.
#
#  [*rabbitmq_use_ssl*]
#    A boolean flag telling whether SSL should be used for communicating with
#    the RabbitMQ service (`true`) or not (`false`).
#    If this is `false` then `rabbitmq_keystore_password`,
#    `rabbitmq_keystore_path`, `rabbitmq_truststore_password` and
#    `rabbitmq_truststore_path` are ignored.
#
#  [*rabbitmq_keystore_path*]
#    Path to Java key-store containing key/certificate pair (PKCS12 format)
#    used to connect to the RabbitMQ service using SSL.
#    Used only if `rabbitmq_use_ssl` is set to `true`.
#
#  [*rabbitmq_keystore_password*]
#    Password for Java key-store containing key/certificate pair used
#    to connect to the RabbitMQ service.
#    Used only if `rabbitmq_use_ssl` is set to `true`.
#
#  [*rabbitmq_truststore_path*]
#    Path to Java key-store containing the X509 certificate of the trusted
#    RabbitMQ service.
#    Used only if `rabbitmq_use_ssl` is set to `true`.
#
#  [*rabbitmq_truststore_password*]
#    Password for Java key-store containing X509 certificate of RabbitMQ
#    service.
#    Used only if `rabbitmq_use_ssl` is set to `true`.
#
#  [*log_max_file_size*]
#    (Optional) Maximum log file size in bytes at which the file will be rotated.
#    Can be followed by optional suffixes 'KB', 'MB', 'GB'. Default is `30MB`.
#
#  [*log_days_max_history*]
#    Maximum number of days to keep previous log files.
#
#  [*service_ensure*]
#    Value to be passed to the service `ensure` attribute. Acceptable values
#    are 'running', 'stopped' or 'undef'.
#    Default is 'running'
#
#  [*enable_hsts*]
#    (Optional) Enable HSTS headers. Default is `false`.
#
#  [*enable_hsts_subdomains*]
#    (Optional) Enable HSTS headers for subdomains of main access server name.
#    Relevant only if `enable_hsts` is `true`. Default is `false`.
#
#  [*custom_properties*]
#    Map of custom server properties to be added to the config.properties file
#
# Actions:
#
# Requires:
#
# Sample Usage:
#
class safeshare::access_service(
  $version                          = $safeshare::access_service::params::version,
  $custom_app_source_path           = $safeshare::access_service::params::custom_app_source_path,
  $custom_notifications_source_path = $safeshare::access_service::params::custom_notifications_source_path,
  $database_hostname                = $safeshare::access_service::params::database_hostname,
  $database_username                = $safeshare::access_service::params::database_username,
  $database_password                = $safeshare::access_service::params::database_password,
  $access_service_url               = $safeshare::access_service::params::access_service_url,
  $webapp_service_url               = $safeshare::access_service::params::webapp_service_url,
  $content_service_url              = $safeshare::access_service::params::content_service_url,
  $import_allow_password            = $safeshare::access_service::params::import_allow_password,
  $mail_auth                        = $safeshare::access_service::params::mail_auth,
  $mail_password                    = $safeshare::access_service::params::mail_password,
  $mail_username                    = $safeshare::access_service::params::mail_username,
  $mail_hostname                    = $safeshare::access_service::params::mail_hostname,
  $mail_starttls_enabled            = $safeshare::access_service::params::mail_starttls_enabled,
  $mail_ssl_enabled                 = $safeshare::access_service::params::mail_ssl_enabled,
  $mail_port                        = $safeshare::access_service::params::mail_port,
  $mail_from_address                = $safeshare::access_service::params::mail_from_address,
  $java_xmx                         = $safeshare::access_service::params::java_xmx,
  $min_firefox_version              = $safeshare::access_service::params::min_firefox_version,
  $company_name                     = $safeshare::access_service::params::company_name,
  $master_encryption_key            = $safeshare::access_service::params::master_encryption_key,
  $bearer_signing_key               = $safeshare::access_service::params::bearer_signing_key,
  $jwt_signing_key                  = $safeshare::access_service::params::jwt_signing_key,
  $jwt_rsa_public_key               = $safeshare::access_service::params::jwt_rsa_public_key,
  $jwt_rsa_private_key              = $safeshare::access_service::params::jwt_rsa_private_key,
  $http_port                        = $safeshare::access_service::params::http_port,
  $rabbitmq_user_name               = $safeshare::access_service::params::rabbitmq_user_name,
  $rabbitmq_password                = $safeshare::access_service::params::rabbitmq_password,
  $rabbitmq_host                    = $safeshare::access_service::params::rabbitmq_host,
  $rabbitmq_port                    = $safeshare::access_service::params::rabbitmq_port,
  $rabbitmq_use_ssl                 = $safeshare::access_service::params::rabbitmq_use_ssl,
  $rabbitmq_keystore_password       = $safeshare::access_service::params::rabbitmq_keystore_password,
  $rabbitmq_keystore_path           = $safeshare::access_service::params::rabbitmq_keystore_path,
  $rabbitmq_truststore_password     = $safeshare::access_service::params::rabbitmq_truststore_password,
  $rabbitmq_truststore_path         = $safeshare::access_service::params::rabbitmq_truststore_path,
  $log_max_file_size                = $safeshare::access_service::params::log_max_file_size,
  $log_days_max_history             = $safeshare::access_service::params::log_days_max_history,
  $service_ensure                   = $safeshare::access_service::params::service_ensure,
  $enable_hsts                      = $safeshare::access_service::params::enable_hsts,
  $enable_hsts_subdomains           = $safeshare::access_service::params::enable_hsts_subdomains,
  $custom_properties                = $safeshare::access_service::params::custom_properties,
) inherits safeshare::access_service::params {

  if $database_hostname == undef {
    fail('database_hostname needs to be defined')
  }

  if $database_username == undef {
    fail('database_username needs to be defined')
  }

  if $database_password == undef {
    fail('database_password needs to be defined')
  }

  if $access_service_url == undef {
    fail('access_service_url needs to be defined')
  }

  if $content_service_url == undef {
    fail('content_service_url needs to be defined')
  }

  validate_bool($mail_auth)

  if $mail_auth {
    if $mail_password == undef {
      fail('mail_auth is enabled so mail_password needs to be defined')
    }

    if $mail_username == undef {
      fail('mail_username needs to be defined')
    }
  }

  if $mail_hostname == undef {
    fail('mail_hostname needs to be defined')
  }

  validate_bool($mail_ssl_enabled)

  validate_bool($mail_starttls_enabled)

  if $mail_ssl_enabled and $mail_starttls_enabled {
    fail('mail_ssl_enabled and mail_starttls_enabled are mutually exclusive')
  }

  validate_numeric($mail_port)

  if $mail_from_address == undef {
    fail('mail_from_address needs to be defined')
  }

  if $master_encryption_key == undef {
    fail('master_encryption_key needs to be defined')
  }

  validate_numeric($http_port)

  if $rabbitmq_user_name == undef {
    fail('rabbitmq_user_name needs to be defined')
  }

  if $rabbitmq_host == undef {
    fail('rabbitmq_host needs to be defined')
  }

  validate_bool($rabbitmq_use_ssl)

  if $rabbitmq_use_ssl {
    if $rabbitmq_keystore_path == undef {
      fail('rabbitmq_use_ssl is enabled so rabbitmq_keystore_path needs to be defined')
    }
    if $rabbitmq_keystore_password == undef {
      fail('rabbitmq_use_ssl is enabled so rabbitmq_keystore_password needs to be defined')
    }

    if $rabbitmq_truststore_path == undef {
      fail('rabbitmq_use_ssl is enabled so rabbitmq_truststore_path needs to be defined')
    }
    if $rabbitmq_truststore_password == undef {
      fail('rabbitmq_use_ssl is enabled so rabbitmq_truststore_password needs to be defined')
    }
  }

  validate_numeric($rabbitmq_port)

  validate_re($log_max_file_size, '^[0-9]*([KMG]B)?$',
    'log_max_file_size must be numeric with option "KB" or "MB" or "GB" suffix')

  validate_numeric($log_days_max_history)

  if $jwt_rsa_public_key == undef {
    fail('jwt_rsa_public_key needs to be defined')
  }

  if $jwt_rsa_private_key == undef {
    fail('jwt_rsa_private_key needs to be defined')
  }

  validate_bool($enable_hsts)

  validate_bool($enable_hsts_subdomains)

  anchor {'safeshare::access_service::begin': } ->
    class { 'safeshare::access_service::install': } ->
    class { 'safeshare::access_service::config': } ->
    class { 'safeshare::access_service::custom': } ->
    class { 'safeshare::access_service::service': } ->
  anchor {'safeshare::access_service::end': }
}
